/*
 * Copyright: (C) 2003 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@tns.net
 *
 */


#include "mileage.h"
#include "mileagefile.h"
#include "mileagecfg.h"
#include "milekeys.h"
#include "mileimport.h"
#include "mileagegraph.h"
#include "mileagegraphdata.h"

#include <qlineedit.h>
#include <qmessagebox.h>
#include <qdatetime.h>
#include <qcombobox.h>
#include <qlistview.h>
#include <qstringlist.h>
#include <qtextstream.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qdir.h>
#include <qmultilineedit.h>
#include <qpointarray.h>

#include <cstdio>
#include <cstdlib>
#include <iostream>


#define CAR_DATA_CFG  "mileage.cfg"


// Constructor
mileage::mileage( QWidget* parent,  const char* name, WFlags fl )
    : mainForm( parent, name, fl )
{
   _current_car = NULL;
   _keys = NULL;
   _import = NULL;
   _units = 0;
   _cost = 0;
   _list.setAutoDelete(TRUE);

   _graph = new mileagegraph(_list, Graph, "graphFrame");
   _graph->setGeometry( QRect( 0, 45, 238, 210 ) );
   //_graph->setFrameShape( QFrame::StyledPanel );
   //_graph->setFrameShadow( QFrame::Raised );
   // Set the default selection for Type of display
   _graph->currentGraph2Changed(graphSelector2->currentText());

   lastText->setReadOnly(TRUE);
   lifetimeText->setReadOnly(TRUE);
   milesMonthText->setReadOnly(TRUE);
   alertEdit->setReadOnly(TRUE);

   // Preset care menu to Activity
   enterActivityButton();

   // Initialize Trip tab
   initTrip();

   // Select proper Units
   updateUnits();

   // Initialize all menus for current Car
   updateCars();
}

// Destructor
mileage::~mileage()
{
   delete _current_car;
   delete _keys;
   delete _import;
   delete _graph;
}


// This function fills all the Car selector menus with the list of cars. It
// then makes a call to process all data for the car that is displayed.
void
mileage::updateCars()
{
   QStringList  str_list;


   // Get the list of all cars
   mileagefile::returnCars(str_list);

   // Clear all menus of all data
   carSelector->clear();
   carSelector_2->clear();
   carSelector_3->clear();
   carSelector_4->clear();
   carSelector_5->clear();
   carSelector_6->clear();

   if (!str_list.isEmpty())
   {
      // Insert car strings into menu
      carSelector->insertStringList(str_list);
      carSelector_2->insertStringList(str_list);
      carSelector_3->insertStringList(str_list);
      carSelector_4->insertStringList(str_list);
      carSelector_5->insertStringList(str_list);
      carSelector_6->insertStringList(str_list);

      // process the default item
      currentCarChanged(carSelector->currentItem());
   }
   else
   {
      // delete current car if no cars displayed
      delete _current_car;
      _current_car = NULL;
   }

}


// This function process all gas mileage related data. It will fill the
// spreadsheet with data for the current selected car. It also will compute
// any performance data for the selected car. If update_tab is set then the
// Display Tab will be update for Mileage.
void
mileage::updateGasData(bool update_tab)
{
   QListViewItem  *last_item = NULL;
   QString  p_str2;
   QString  last_mileage;
   int      total_odo = 0;
   double   total_fuel = 0.0;
   QString  str5, str6;
   bool     first_entry = TRUE;
   QDate    first_date, last_date;
   int      first_odo, last_odo;
   int      mm, dd, yyyy;
   QChar    dummy;
   double   miles_month = 0.0;
   bool     first_update = TRUE;


   if (update_tab)
   {
      gasData->setColumnText(0, "Date");
      gasData->setColumnText(1, "Odometer");
      gasData->setColumnText(4, "Cost");
      gasData->setColumnText(5, "");
      gasData->setColumnText(6, "");
      // Set correct labels for gas data
      switch (_units)
      {
         // MPG
         case 0:
            gasData->setColumnText(2, "Gallons");
            gasData->setColumnText(3, "MPG");
         break;

         // L/100km
         case 1:
            gasData->setColumnText(2, "Liters");
            gasData->setColumnText(3, "L/100km");
         break;

         // L/10km
         case 2:
            gasData->setColumnText(2, "Liters");
            gasData->setColumnText(3, "L/10km");
         break;

         // Miles / L
         case 3:
            gasData->setColumnText(2, "Liters");
            gasData->setColumnText(3, "M/L");
         break;

         // km / L
         case 4:
            gasData->setColumnText(2, "Liters");
            gasData->setColumnText(3, "km/L");
         break;
      }
   }

   // For the currently selected car
   if (_current_car)
   {
      QStringList  str_list;

      // Get all the mile entries for this car
      _current_car->returnMileEntries(str_list);

      // Sort list by ascending date
      str_list.sort();

      // Process each entry
      for (QStringList::Iterator it = str_list.begin(); it != str_list.end(); it++)
      {
         QListViewItem  *item;

         // Read in date, odometer, and fuel
         QTextStream  ts(&(*it), IO_ReadOnly);
         QString  str1, str2, str3, str4, end_str, p;
         ts >> str1 >> str2 >> str3 >> end_str;

         // See if there is cost data for this entry. If there is then end_str
         if (ts.atEnd())
         {
            p = end_str;
            end_str.truncate(0);
         }
         else
            ts >> p;

         if (first_entry == TRUE)
         {
            first_entry = FALSE;
            QTextStream ts1(str1, IO_ReadOnly);
            QTextStream ts2(str2, IO_ReadOnly);

            ts2 >> first_odo;

            // Get the year, month, and day in number format
            ts1 >> dec >> yyyy >> dummy >> mm >> dummy >> dd;
            first_date.setYMD(yyyy, mm, dd);
         }

         QTextStream ts1(str1, IO_ReadOnly);
         QTextStream ts2(str2, IO_ReadOnly);

         ts2 >> last_odo;

         // Get the year, month, and day in number format
         ts1 >> dec >> yyyy >> dummy >> mm >> dummy >> dd;
         last_date.setYMD(yyyy, mm, dd);

         // IF there is a previous item then calculate mileage
         if (!first_update)
         {
            QTextStream  old_odo(p_str2, IO_ReadOnly);
            QTextStream  new_odo(str2, IO_ReadOnly);
            QTextStream  new_fuel(str3, IO_ReadOnly);
            int      old_odo_d;
            int      new_odo_d;
            double   new_fuel_d;
            double   mileage_d;

            old_odo >> old_odo_d;
            new_odo >> new_odo_d;
            new_fuel >> new_fuel_d;

            if (p == "0")
            {
               total_odo += (new_odo_d - old_odo_d);
               total_fuel += new_fuel_d;

               // MPG
               if (_units == 0)
                  mileage_d = (double)(new_odo_d - old_odo_d) / new_fuel_d;
               // L/100km
               else if (_units == 1)
                  mileage_d = 100.0 * new_fuel_d / (double)(new_odo_d - old_odo_d);
               // L/10km
               else if (_units == 2)
                  mileage_d = 10.0 * new_fuel_d / (double)(new_odo_d - old_odo_d);
               // Miles / L
               else if (_units == 3)
                  mileage_d = (double)(new_odo_d - old_odo_d) / new_fuel_d;
               // km / L
               else if (_units == 4)
                  mileage_d = (double)(new_odo_d - old_odo_d) / new_fuel_d;
               str4.setNum(mileage_d, 'g', 4);
            }
         }

         if (update_tab)
         {
            // Add this item to the spreadsheet
            item = new QListViewItem(gasData,
               last_item, str1, str2, str3, str4, end_str);
            last_item = item;
         }

         // Save off data needed
         first_update = FALSE;
         p_str2 = str2;
         last_mileage = str4;
      }

      // If there is data to calculate lifetime MPG
      if (total_fuel > 0.0)
      {
         // MPG
         if (_units == 0)
            total_fuel = (double)total_odo / total_fuel;
         // L/100km
         else if (_units == 1)
            total_fuel = 100.0 * total_fuel / (double)total_odo;
         // L/10km
         else if (_units == 2)
            total_fuel = 10.0 * total_fuel / (double)total_odo;
         // Miles / L
         else if (_units == 3)
            total_fuel = (double)total_odo / total_fuel;
         // km / L
         else if (_units == 4)
            total_fuel = (double)total_odo / total_fuel;
         
         str5.setNum(total_fuel, 'g', 4);
      }

      if (first_date != last_date)
      {
         miles_month = (last_odo - first_odo) * 30.0 / first_date.daysTo(last_date);
         str6.setNum(miles_month, 'g', 4);
      }

      // Update MPG entries on Car tab
      lastText->validateAndSet(last_mileage, 0, 0, 0);
      lifetimeText->validateAndSet(str5, 0, 0, 0);
      milesMonthText->validateAndSet(str6, 0, 0, 0);
   }
}


void
mileage::updateActivityData()
{
   QListViewItem  *last_item = NULL;


   gasData->setColumnText(0, "Date");
   gasData->setColumnText(1, "Odometer");
   gasData->setColumnText(2, "Activity");
   gasData->setColumnText(3, "Comment");
   gasData->setColumnText(4, "");
   gasData->setColumnText(5, "");
   gasData->setColumnText(6, "");

   // For the currently selected car
   if (_current_car)
   {
      QStringList  str_list;

      // Get all the mile entries for this car
      _current_car->returnActivityEntries(str_list);

      // Sort list by ascending date
      str_list.sort();

      // Process each entry
      for (QStringList::Iterator it = str_list.begin(); it != str_list.end(); it++)
      {
         QListViewItem  *item;
         QChar  temp;
         QTextStream  ts(&(*it), IO_ReadOnly);
         QString  date_str, odo_str, act_str, com_str;
         QString  len1, len2;
         QTextStream  ts_len1(len1, IO_ReadOnly);
         QTextStream  ts_len2(len2, IO_ReadOnly);
         int int_len1, int_len2;

         // Read in date, odometer, and fuel
         ts >> date_str >> odo_str >> len1;
         ts_len1 >> int_len1;
         ts >> ws;

         while (int_len1)
         {
            ts >> temp;
            act_str.append(temp);
            int_len1--;
         }

         ts >> len2;
         ts_len2 >> int_len2;
         ts >> ws;
         while (int_len2)
         {
            ts >> temp;
            com_str.append(temp);
            int_len2--;
         }

         // Add this item to the spreadsheet
         item = new QListViewItem(gasData, last_item,
            date_str, odo_str, act_str, com_str);
         last_item = item;
      }
   }
}


void
mileage::updateAlertData()
{
   QListViewItem  *last_item = NULL;


   gasData->setColumnText(0, "Odometer");
   gasData->setColumnText(1, "Repeat Every");
   gasData->setColumnText(2, "Activity");
   gasData->setColumnText(3, "Comment");
   gasData->setColumnText(4, "");
   gasData->setColumnText(5, "");
   gasData->setColumnText(6, "");

   // For the currently selected car
   if (_current_car)
   {
      QStringList  str_list;

      // Get all the mile entries for this car
      _current_car->returnAlertEntries(str_list);

      // Sort list by ascending date
      str_list.sort();

      // Process each entry
      for (QStringList::Iterator it = str_list.begin(); it != str_list.end(); it++)
      {
         QListViewItem  *item;
         QChar  temp;
         QTextStream  ts(&(*it), IO_ReadOnly);
         QString  odo_str, rep_str, act_str, com_str;
         QString  len1, len2;
         QTextStream  ts_len1(len1, IO_ReadOnly);
         QTextStream  ts_len2(len2, IO_ReadOnly);
         int int_len1, int_len2;

         // Read in date, odometer, and fuel
         ts >> odo_str >> rep_str >> len1;
         ts_len1 >> int_len1;
         ts >> ws;

         while (int_len1)
         {
            ts >> temp;
            act_str.append(temp);
            int_len1--;
         }

         ts >> len2;
         ts_len2 >> int_len2;
         ts >> ws;
         while (int_len2)
         {
            ts >> temp;
            com_str.append(temp);
            int_len2--;
         }

         // Add this item to the spreadsheet
         item = new QListViewItem(gasData, last_item,
            odo_str, rep_str, act_str, com_str);
         last_item = item;
      }
   }
}


void
mileage::updateTripData()
{
   QListViewItem  *last_item = NULL;


   gasData->setColumnText(0, "Start Date");
   gasData->setColumnText(1, "Start Odometer");
   gasData->setColumnText(2, "End Date");
   gasData->setColumnText(3, "End Odometer");
   gasData->setColumnText(4, "Person");
   gasData->setColumnText(5, "Destination");
   gasData->setColumnText(6, "Purpose");

   // For the currently selected car
   if (_current_car)
   {
      QStringList  str_list;

      // Get all the trip entries for this car
      _current_car->returnTripEntries(str_list);

      // Sort list by ascending date
      str_list.sort();

      // Process each entry
      for (QStringList::Iterator it = str_list.begin(); it != str_list.end(); it++)
      {
         QListViewItem  *item;
         QChar  temp;
         QTextStream  ts(&(*it), IO_ReadOnly);
         QString  s_d_str, s_m_str, e_d_str, e_m_str;
         QString  person, dest, purpose;
         QString  len1, len2, len3;
         QString  len4, len5, len6, len7;
         QTextStream  ts_len1(len1, IO_ReadOnly);
         QTextStream  ts_len2(len2, IO_ReadOnly);
         QTextStream  ts_len3(len3, IO_ReadOnly);
         QTextStream  ts_len4(len4, IO_ReadOnly);
         QTextStream  ts_len5(len5, IO_ReadOnly);
         QTextStream  ts_len6(len6, IO_ReadOnly);
         QTextStream  ts_len7(len7, IO_ReadOnly);
         int int_len1, int_len2, int_len3;
         int int_len4, int_len5, int_len6, int_len7;

         ts >> len1;
         ts_len1 >> int_len1;
         ts >> ws;
         while (int_len1)
         {
            ts >> temp;
            s_d_str.append(temp);
            int_len1--;
         }

         ts >> len2;
         ts_len2 >> int_len2;
         ts >> ws;
         while (int_len2)
         {
            ts >> temp;
            s_m_str.append(temp);
            int_len2--;
         }

         ts >> len3;
         ts_len3 >> int_len3;
         ts >> ws;
         while (int_len3)
         {
            ts >> temp;
            e_d_str.append(temp);
            int_len3--;
         }

         ts >> len4;
         ts_len4 >> int_len4;
         ts >> ws;
         while (int_len4)
         {
            ts >> temp;
            e_m_str.append(temp);
            int_len4--;
         }

         ts >> len5;
         ts_len5 >> int_len5;
         ts >> ws;
         while (int_len5)
         {
            ts >> temp;
            person.append(temp);
            int_len5--;
         }

         ts >> len6;
         ts_len6 >> int_len6;
         ts >> ws;
         while (int_len6)
         {
            ts >> temp;
            dest.append(temp);
            int_len6--;
         }

         ts >> len7;
         ts_len7 >> int_len7;
         ts >> ws;
         while (int_len7)
         {
            ts >> temp;
            purpose.append(temp);
            int_len7--;
         }

         // Add this item to the spreadsheet
         item = new QListViewItem(gasData, last_item,
            s_d_str, s_m_str, e_d_str, e_m_str, person, dest, purpose);
         last_item = item;
      }
   }
}


// This function will cause all Car selector menus to be syncronized together.
// When one is changed this function will cause all of them to show the same
// car. Also an object for the car's file will be instantiated and Gas Data
// for this car processed.
void
mileage::currentCarChanged(int item)
{
   QStringList  str_list;


   // Sync all car menus together
   carSelector->setCurrentItem(item);
   carSelector_2->setCurrentItem(item);
   carSelector_3->setCurrentItem(item);
   carSelector_4->setCurrentItem(item);
   carSelector_5->setCurrentItem(item);
   carSelector_6->setCurrentItem(item);

   // Delete previous car and create new car
   delete _current_car;
   _current_car =
      new mileagefile(carSelector->text(carSelector->currentItem()));

   // Process this entry
   updateDisplayTab();

   updateAlertDisplay();

   // Get all the mile entries for this car
   _current_car->returnMileEntries(str_list);

   // Sort list by ascending date
   str_list.sort();

   // Remove all previous entries
   while (_list.removeFirst() == TRUE) {}

   // Process each entry
   for (QStringList::Iterator it = str_list.begin(); it != str_list.end(); it++)
   {
      mileagegraphdata *data = new mileagegraphdata();
      data->setAllData((*it));
      _list.append(data);
   }

   // Update graph with new data
   _graph->currentCarChanged(graphSelector);
}


void
mileage::currentGraphChanged(const QString& str)
{
   _graph->currentGraphChanged(str);
   _graph->raise();
   _graph->update();
}


void
mileage::currentGraph2Changed(const QString& str)
{
   _graph->currentGraph2Changed(str);
   _graph->raise();
   _graph->update();
}


void
mileage::updateAlertDisplay()
{
   QStringList  lst;


   // Get the next alert for this car
   if (_current_car)
   {
      alertEdit->clear();
      _current_car->getNextAlert(1000, lst);
      if (lst.count() == 0)
         alertEdit->insertLine("No Current Alerts for this car", 0);
      else
      {
         int count = 0;
         alertEdit->insertLine("Alerts for this car are:", count++);
         lst.sort();
         for (QStringList::Iterator it = lst.begin(); it != lst.end(); ++it)
         {
            QTextStream  ts(&(*it), IO_ReadOnly);
            QString  s1, odo_str, act_str, com_str;
            QChar  temp;
            int  act_num, com_num;
            ts >> odo_str >> act_num;
            ts >> ws;

            while (act_num)
            {
               ts >> temp;
               act_str.append(temp);
               act_num--;
            }

            ts >> com_num;
            ts >> ws;
            while (com_num)
            {
               ts >> temp;
               com_str.append(temp);
               com_num--;
            }

            s1.append(act_str);
            s1.append(" due at ");
            s1.append(odo_str);
            s1.append(": ");
            s1.append(com_str);
            alertEdit->insertLine(s1, count++);
         }
      }
   }
}


// This is called when the user presses the new car button. It will check for
// a duplicate entry and for an empty entry.
void
mileage::enterNewCar()
{
   QString text = newCarText->text();
   if (text.isEmpty())   
   {
      QMessageBox::information(this, "Warning", "Must enter a name for the car.",
         QMessageBox::Ok);
   }
   else
   {
      if (mileagefile::searchForCar(text) == FALSE)
      {
         if (mileagefile::createNewCar(text) == FALSE)
         {
            QMessageBox::information(this,
               "Unable to create Car.", text, QMessageBox::Ok);
         }
         else
         {
            newCarText->clear();
         }
         updateCars();
      }
      else
      {
         QMessageBox::information(this,
            "This car already entered.", text, QMessageBox::Ok);
      }
   }
}


// This is called when the user presses the Import car button. It will check for
// a duplicate entry and for an empty entry.
void
mileage::importDataCar()
{
   QString text = newCarText->text();
   if (text.isEmpty())
   {
      QMessageBox::information(this, "Warning", "Must enter a name for the car.",
         QMessageBox::Ok);
   }
   else
   {
      if (mileagefile::searchForCar(text) == FALSE)
      {
         if (_import == NULL)
            _import = new mileimport();
         _import->show();
      }
      else
      {
         QMessageBox::information(this,
            "This car already entered.", text, QMessageBox::Ok);
      }
   }
}


// This will delete a car completely from the program. It will prompt the
// user to make sure they want to delete the car.
void
mileage::deleteCar()
{
   QString  str;
   QString  car;

   if (carSelector->count() != 0)
   {
      // Display msg asking user if they are sure they want to delete
      car = carSelector->text(carSelector->currentItem());
      str.append("This will delete the car\n");
      str.append(car);
      str.append("\nThis will remove all data for this car,\n Are you sure?");

      switch( QMessageBox::warning( this, "Delete",
                          str,
                          "Yes, Delete", "No, Do Not Delete", 0,
                          0, 1 ))
      {
         case 0: // Yes
            mileagefile::deleteCar(car);

            // Update for the new selected car
            updateCars();
            break;
         case 1: // No
            break;
      }
   }

}


// This is called when the user presses the Enter Today's Date button
void
mileage::enterTodayDate()
{
   QDate date = QDate::currentDate();
   QString str = QString::number(date.year()) + "/" +
                 QString::number(date.month()) + "/" +
                 QString::number(date.day());
   newDateText->validateAndSet(str, 0, 0, 0);
   newDateText->cursorRight(0, str.length());
}


// This function is called when the user selects the Submit Entry button. It
// will validate user entry and if OK will store this mileage data.
void
mileage::submitEntry()
{
   QString date_str = newDateText->text();
   QString odo_str  = odometerText->text();
   QString fuel_str = fuelText->text();
   QString cost_str = costText->text();
   QString date_new;
   bool odo_flag, fuel_flag;
   bool check_date;


   //TBD maybe odometer should be double
   (void)odo_str.toUInt(&odo_flag);
   (void)fuel_str.toDouble(&fuel_flag);

   check_date = checkDate(date_str, date_new);
   QString total_str = date_new + " " + odo_str + " " + fuel_str + " " + cost_str;

   // IF not cars are selected
   if (!_current_car)
   {
      QMessageBox::information(this, "Warning",
         "Please create a car first.", QMessageBox::Ok);
   }
   // Check for a valid date
   else if (date_str.isEmpty() || !check_date)
   {
      QMessageBox::information(this, "Warning",
         "Invalid Date.", QMessageBox::Ok);
   }
   // Check for presense of string and a number
   else if (odo_str.isEmpty() || !odo_flag)
   {
      QMessageBox::information(this, "Warning",
         "Invalid Odometer.", QMessageBox::Ok);
   }
   // Check for presense of string and a number
   else if (fuel_str.isEmpty() || !fuel_flag)
   {
      QMessageBox::information(this, "Warning",
         "Invalid Fuel.", QMessageBox::Ok);
   }
   else
   {
      // IF a car is selected then add this entry and clear fields
      if (_current_car)
      {
         _current_car->addMileEntry(total_str, previousButton->isChecked());
         newDateText->clear();
         odometerText->clear();
         fuelText->clear();
         costText->clear();
         previousButton->setChecked(FALSE);;
      }
      updateDisplayTab();
   }
}


void
mileage::submitAlert()
{
   bool odo_flag;


   QString date_str  = activityDate->text();
   QString odo_str  = odoActivity->text();
   QString rep_str  = activityRepeat->text();
   QString act_str  = activityText->text();
   QString com_str  = activityComment->text();
   (void)odo_str.toUInt(&odo_flag);

   // Process Alert
   if (alertButton->isChecked())
   {
      // Check for presense of string and a number
      if (odo_str.isEmpty() || !odo_flag)
      {
         QMessageBox::information(this, "Warning",
            "Invalid Odometer.", QMessageBox::Ok);
      }
      else if (act_str.isEmpty())
      {
         QMessageBox::information(this, "Warning",
            "Empty Activity.", QMessageBox::Ok);
      }
      else
      {
         // IF a car is selected then add this Alert and clear fields
         if (_current_car)
         {
            // IF no repeat then put 0
            if (rep_str.isEmpty())
               rep_str = "0";

            _current_car->addAlertEntry(odo_str, rep_str, act_str, com_str);
            activityDate->clear();
            odoActivity->clear();
            activityRepeat->clear();
            activityText->clear();
            activityComment->clear();
         }
         updateDisplayTab();
         updateAlertDisplay();
      }
   }
   // Process Activity
   else
   {
      QString date_new;
      bool  check_date;

      check_date = checkDate(date_str, date_new);

      if (act_str.isEmpty())
      {
         QMessageBox::information(this, "Warning",
            "Empty Activity.", QMessageBox::Ok);
      }
      // Check for presense of string and a number
      else if (odo_str.isEmpty() || !odo_flag)
      {
         QMessageBox::information(this, "Warning",
            "Invalid Odometer.", QMessageBox::Ok);
      }
      else if (date_str.isEmpty() || !check_date)
      {
         QMessageBox::information(this, "Warning",
            "Invalid Date.", QMessageBox::Ok);
      }
      else
      {
         // IF a car is selected then add this Activity and clear fields
         if (_current_car)
         {
            _current_car->addActivityEntry(date_new, odo_str, act_str, com_str);
            activityDate->clear();
            odoActivity->clear();
            activityRepeat->clear();
            activityText->clear();
            activityComment->clear();
         }
         updateDisplayTab();
      }
   }
}


// Called when user presses the delete entry button
void
mileage::deleteEntry()
{
   QListViewItem *item;


   item = gasData->selectedItem();
   if (item == NULL)
   {
      QMessageBox::information(this, "Warning",
         "Please select item.", QMessageBox::Ok);
   }
   else
   {
      switch( QMessageBox::warning( this, "Delete",
                          "Are you sure you want\nto delete this entry?",
                          "Yes, Delete", "No, Do Not Delete", 0,
                          0, 1 ))
      {
         case 0: // Yes
            if (displaySelector->currentText() == "Display Mileage")
            {
               // Delete mileage entry for selected car
               if (_current_car)
                  _current_car->deleteEntry(item->text(1));
            }
            else if (displaySelector->currentText() == "Display Activity")
            {
               // Delete activity entry for selected car
               if (_current_car)
                  _current_car->deleteActivity(item->text(0),
                                               item->text(1),
                                               item->text(2));
            }
            else if (displaySelector->currentText() == "Display Alerts")
            {
               // Delete alert entry for selected car
               if (_current_car)
                  _current_car->deleteAlert(item->text(0),
                                            item->text(1),
                                            item->text(2));
            }
            else if (displaySelector->currentText() == "Display Trips")
            {
               // Delete trip entry for selected car
               if (_current_car)
                  _current_car->deleteTrip(item->text(1));
            }
            break;
         case 1: // No
            break;
      }
   }
   updateDisplayTab();
   updateAlertDisplay();
}

// Called when user presses 0 on the GUI button
void
mileage::enter0Button()
{
   enterKeyPress("0");
}

// Called when user presses 1 on the GUI button
void
mileage::enter1Button()
{
   enterKeyPress("1");
}

// Called when user presses 2 on the GUI button
void
mileage::enter2Button()
{
   enterKeyPress("2");
}

// Called when user presses 3 on the GUI button
void
mileage::enter3Button()
{
   enterKeyPress("3");
}

// Called when user presses 4 on the GUI button
void
mileage::enter4Button()
{
   enterKeyPress("4");
}

// Called when user presses 5 on the GUI button
void
mileage::enter5Button()
{
   enterKeyPress("5");
}

// Called when user presses 6 on the GUI button
void
mileage::enter6Button()
{
   enterKeyPress("6");
}

// Called when user presses 7 on the GUI button
void
mileage::enter7Button()
{
   enterKeyPress("7");
}

// Called when user presses 8 on the GUI button
void
mileage::enter8Button()
{
   enterKeyPress("8");
}

// Called when user presses 9 on the GUI button
void
mileage::enter9Button()
{
   enterKeyPress("9");
}

// Called when user presses . on the GUI button
void
mileage::enterDotButton()
{
   enterKeyPress(".");
}

// Called when user presses / on the GUI button
void
mileage::enterSlashButton()
{
   enterKeyPress("/");
}

// Called when user wants to delete a character
void
mileage::backspaceCallback()
{
   enterKeyPress("backspace");
}

void
mileage::enterAlertButton()
{
   alertButton->setChecked(TRUE);
   activityButton->setChecked(FALSE);
   activityRepeat->setEnabled(TRUE);
   repeatEveryText->setEnabled(TRUE);
   careDateLabel->setEnabled(FALSE);
   activityDate->setEnabled(FALSE);
}


void
mileage::enterActivityButton()
{
   alertButton->setChecked(FALSE);
   activityButton->setChecked(TRUE);
   activityRepeat->setEnabled(FALSE);
   repeatEveryText->setEnabled(FALSE);
   careDateLabel->setEnabled(TRUE);
   activityDate->setEnabled(TRUE);
}


// This is called when a user presses the key pad. It checks for which text
// box has focus to enter data there.
void
mileage::enterKeyPress(QString str)
{
   QLineEdit *focusText = NULL;

   // Determine what text box has focus
   if (newDateText->hasFocus())
      focusText = newDateText;
   else if (odometerText->hasFocus())
      focusText = odometerText;
   else if (fuelText->hasFocus())
      focusText = fuelText;
   else if (costText->hasFocus())
      focusText = costText;
   else
   {
      QMessageBox::information(this, "Warning", "Please select a text field.",
         QMessageBox::Ok);
   }

   if (focusText)
   {
      if (str == "backspace")
      {
         focusText->backspace();
      }
      else
      {
         // Enter key press into text box at end of text
         QString new_str = focusText->text() + str;
         focusText->validateAndSet(new_str, 0, 0, 0);
         focusText->cursorRight(0, new_str.length());
      }
   }
}


// Switches displayed units
void
mileage::unitsSelected(int unit)
{
   QString dir_str;
   QString str;


   _units = unit;

   switch (unit)
   {
      // MPG
      case 0:
         lifetimeLabel->setText("Lifetime MPG");
         lastLabel->setText("Last MPG");
         milesMonthLabel->setText("Miles / Month");
         fuelLabel->setText("Enter Gallons");
         gasData->setColumnText(2, "Gallons");
         gasData->setColumnText(3, "MPG");
      break;

      // L/100km
      case 1:
         lifetimeLabel->setText("Lifetime L/100km");
         lastLabel->setText("Last L/100km");
         milesMonthLabel->setText("Kilometers / Month");
         fuelLabel->setText("Enter Liters");
         gasData->setColumnText(2, "Liters");
         gasData->setColumnText(3, "L/100km");
      break;

      // L/10km
      case 2:
         lifetimeLabel->setText("Lifetime L/10km");
         lastLabel->setText("Last L/10km");
         milesMonthLabel->setText("Kilometers / Month");
         fuelLabel->setText("Enter Liters");
         gasData->setColumnText(2, "Liters");
         gasData->setColumnText(3, "L/10km");
      break;

      // Miles / L
      case 3:
         lifetimeLabel->setText("Lifetime Miles/L");
         lastLabel->setText("Last Miles/L");
         milesMonthLabel->setText("Miles / Month");
         fuelLabel->setText("Enter Liters");
         gasData->setColumnText(2, "Liters");
         gasData->setColumnText(3, "M/L");
      break;

      // km / L
      case 4:
         lifetimeLabel->setText("Lifetime km/L");
         lastLabel->setText("Last km/L");
         milesMonthLabel->setText("Kilometers / Month");
         fuelLabel->setText("Enter Liters");
         gasData->setColumnText(2, "Liters");
         gasData->setColumnText(3, "km/L");
      break;

   }

   _graph->setUnits(_units, _cost);

   // Write out units
   mileagecfg * cfg = new mileagecfg();
   cfg->setUnits(_units);
   delete cfg;
   
   updateDisplayTab();
}


void
mileage::costSelected(int cost)
{
   _cost = cost;

   _graph->setUnits(_units, _cost);

   // Write out cost
   mileagecfg * cfg = new mileagecfg();
   cfg->setCostUnits(_cost);
   delete cfg;

   updateDisplayTab();
}


// Update the Units Combo Box with the data saved from a previous run
void
mileage::updateUnits()
{
   QString dir_str;
   QString str;


   // Read units from file
   mileagecfg * cfg = new mileagecfg();
   _units = cfg->returnUnits();
   _cost  = cfg->returnCostUnits();
   delete cfg;

   mpgBox->setCurrentItem(_units);
   costBox->setCurrentItem(_cost);
   unitsSelected(_units);
}


void
mileage::updateDisplayTab()
{
   QString text;


   // Clear the gas data from the spreadsheet
   gasData->clear();


   text = displaySelector->currentText();
   if (text == "Display Mileage")
      updateGasData(TRUE);
   else if (text == "Display Activity")
   {
      updateGasData(FALSE);
      updateActivityData();
   }
   else if (text == "Display Alerts")
   {
      updateGasData(FALSE);
      updateAlertData();
   }
   else if (text == "Display Trips")
   {
      updateGasData(FALSE);
      updateTripData();
   }
}


bool
mileage::checkDate(
   QString& old_date,
   QString& new_date)
{
   int  mm, dd, yyyy;
   QChar  dummy;


   QTextStream ts(old_date, IO_ReadOnly);
   // Get the year, month, and day in number format
   ts >> dec >> yyyy >> dummy >> mm >> dummy >> dd;

   // Add leading 0's so can sort by date correctly
   new_date.sprintf("%04d/%02d/%02d", yyyy, mm, dd);

   return(QDate::isValid(yyyy, mm, dd));
}


void
mileage::initTrip()
{
   QStringList  persons, dests;


   // Get person lists and destination lists
   mileagecfg * cfg = new mileagecfg();
   cfg->returnPersons(persons);
   cfg->returnDestinations(dests);
   delete cfg;

   // Delete any items already in the lists
   personCombo->clear();
   destinationCombo->clear();

   personCombo->insertStringList(persons);
   destinationCombo->insertStringList(dests);
}


// This function will check and store a Trip entry.
void
mileage::processTrip()
{
   QString  start_date;
   QString  start_mile;
   QString  end_date;
   QString  end_mile;
   QString  person;
   QString  dest;
   QString  purpose;
   QString  start_date_new, end_date_new;
   bool     start_date_check, end_date_check;
   bool     start_mile_flag, end_mile_flag;


   start_date = startDateText->text();
   start_mile = startMileText->text();
   end_date = endDateText->text();
   end_mile = endMileText->text();
   person = personCombo->currentText();
   dest = destinationCombo->currentText();
   purpose = purposeText->text();

   start_date_check = checkDate(start_date, start_date_new);
   end_date_check = checkDate(end_date, end_date_new);
   (void)start_mile.toUInt(&start_mile_flag);
   (void)end_mile.toUInt(&end_mile_flag);

   if (start_date.isEmpty() || !start_date_check)
   {
      QMessageBox::information(this, "Warning",
         "Invalid Start Date.", QMessageBox::Ok);
   }
   else if (!end_date.isEmpty() && !end_date_check)
   {
      QMessageBox::information(this, "Warning",
         "Invalid End Date.", QMessageBox::Ok);
   }
   // Check for presense of string and a number
   else if (start_mile.isEmpty() || !start_mile_flag)
   {
      QMessageBox::information(this, "Warning",
         "Invalid Start Odometer.", QMessageBox::Ok);
   }
   // Check for presense of string and a number
   else if (!end_mile.isEmpty() && !end_mile_flag)
   {
      QMessageBox::information(this, "Warning",
         "Invalid End Odometer.", QMessageBox::Ok);
   }
   else
   {
      // Store person and destination
      mileagecfg * cfg = new mileagecfg();
      cfg->setPerson(person);
      cfg->setDestination(dest);
      delete cfg;

      // Add new Trip entry
      _current_car->addTripEntry(start_date_new, start_mile,
         end_date_new, end_mile, person, dest, purpose);

      // Redraw person and destination pulldowns
      initTrip();

      // Make sure we display that last entered values
      personCombo->setEditText(person);
      destinationCombo->setEditText(dest);

      updateDisplayTab();
   }
}


void
mileage::startDateKeypad()
{
   delete _keys;
   _keys = new milekeys(FALSE, startDateText);
   _keys->show();
}


void
mileage::startOdoKeypad()
{
   delete _keys;
   _keys = new milekeys(TRUE, startMileText);
   _keys->show();
}


void
mileage::endDateKeypad()
{
   delete _keys;
   _keys = new milekeys(FALSE, endDateText);
   _keys->show();
}


void
mileage::endOdoKeypad()
{
   delete _keys;
   _keys = new milekeys(TRUE, endMileText);
   _keys->show();
}


void
mileage::tripSelected(QListViewItem *item)
{
   if ((item != NULL) && (displaySelector->currentText() == "Display Trips"))
   {
      // Clear out all fields
      startDateText->clear();
      startMileText->clear();
      endDateText->clear();
      endMileText->clear();
      //personCombo->clear();
      //destinationCombo->clear();
      purposeText->clear();

      // Set all fields
      startDateText->setText(item->text(0));
      startMileText->setText(item->text(1));
      endDateText->setText(item->text(2));
      endMileText->setText(item->text(3));
      //personCombo->insertItem(item->text(4));
      //destinationCombo->insertItem(item->text(5));
      purposeText->setText(item->text(6));

      personCombo->setEditText(item->text(4));
      destinationCombo->setEditText(item->text(5));
   }
}

