/*
 * Copyright: (C) 2003 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@tns.net
 *
 */

#include "mileagefile.h"

#include <qfile.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtextstream.h>

#include <cstdio>
#include <cstdlib>


#define CAR_DATA_EXT   ".car"
#define CAR_NAME_FILTER "*.car"
#define FIRST_LINE_STRING "This is a data file for the Car Mileage program V1.0"

// Below is how the data is defined
// M - Mileage entry, date, odometer, fuel, and optional cost
// P - Mileage entry as M but no previous entry
// A - Name of car
// C - Alert - Odometer, repeat (0 if no repeat), Activity, Comments
// D - Activity - Date, Odometer, Activity, Comments
// R - Person category for Trip tab
// E - Destination category for Trip tab
// F - Trip entry
//


// Constructor
mileagefile::mileagefile(const char *carname)
{
   QString dir_str;

   if (getenv("HOME"))
   {
      dir_str = getenv("HOME");
      dir_str.append("/");
   }
   dir_str.append(CAR_DATA_DIR);
   _dir = new QDir();
   _dir->mkdir(dir_str, TRUE);
   _dir->setFilter(QDir::Files | QDir::NoSymLinks);
   _filestr = new QString();
   _filestr->append(dir_str);
   _filestr->append(carname);
   _filestr->append(CAR_DATA_EXT);
}


// Destructor
mileagefile::~mileagefile()
{
   delete _dir;
   delete _filestr;
}


// Static function that returns a list of available cars
void
mileagefile::returnCars(QStringList & cars)
{
   QString dir_str;
   QDir dir;

   // Find location of car files
   if (getenv("HOME"))
   {
      dir_str = getenv("HOME");
      dir_str.append("/");
   }
   dir_str.append(CAR_DATA_DIR);
   dir.setPath(dir_str);
   dir.mkdir(dir_str, TRUE);
   dir.setFilter(QDir::Files | QDir::NoSymLinks);
   dir.setNameFilter(CAR_NAME_FILTER);

   const QFileInfoList *list = dir.entryInfoList();
   QFileInfoListIterator it( *list );
   QFileInfo *fi;

   // loop thru every file in the directory
   while ( (fi = it.current()) != NULL )
   {
      QFile  filename;
      filename.setName(dir_str + fi->fileName());

      if (filename.open(IO_ReadOnly))
      {
         QTextStream t(&filename);

         QString  str = t.readLine();
         // Find if this is one of our files
         if (str == FIRST_LINE_STRING)
         {
            QChar  chr;
            t >> chr;
            // If this is one of our files then add to car list
            if (chr == "A")
            {
               t.skipWhiteSpace();
               str = t.readLine();
               cars << str;
            }
         }
      }
      filename.close();
      ++it;
   }
}


// Delete car from those available. This will remove the data file as well
void
mileagefile::deleteCar(const char *carname)
{
   QString dir_str;
   QString str;

   // Get string to directory and file
   if (getenv("HOME"))
   {
      dir_str = getenv("HOME");
      dir_str.append("/");
   }
   str.append(dir_str);
   str.append("/");
   str.append(CAR_DATA_DIR);
   str.append(carname);
   str.append(CAR_DATA_EXT);

   QFile  filename(str);

   // Remove file
   filename.remove();
}


// This function deletes a mileage entry in car file
void
mileagefile::deleteEntry(QString str)
{
   QFile o_f(*_filestr);
   QStringList  lst;
   QString  tag, date, odo;

   // Open file to read 
   if (o_f.open(IO_ReadOnly))
   {
      QTextStream t(&o_f);

      // Read all entries
      while (!t.atEnd())
      {
         QString  l_str = t.readLine();
         QChar chr = l_str.at(0);
         if ((chr == "M") || (chr == "P"))
         {
            QTextStream s(&l_str, IO_ReadOnly);
            s >> tag;
            s >> date;
            s >> odo;

            // Dump all entries that are not the correct odometer reading
            if (str != odo)
               lst.append(l_str);
         }
         else
            lst.append(l_str);
      }
      o_f.close();
      o_f.remove();

      // Dump the entries to the original file
      if (o_f.open(IO_WriteOnly))
      {
         QTextStream tt(&o_f);
         for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it )
            tt << (*it) << endl;

         o_f.close();
      }
   }
}


// This function deletes an activity
void
mileagefile::deleteActivity(
   QString  dat,
   QString  odo,
   QString  act)
{
   QFile o_f(*_filestr);
   QStringList  lst;
   QString  f_tag, f_dat, f_odo;
   int      len;

   // Open file to read 
   if (o_f.open(IO_ReadOnly))
   {
      QTextStream t(&o_f);

      // Read all entries
      while (!t.atEnd())
      {
         QString  f_act;
         QString  l_str = t.readLine();
         QChar chr = l_str.at(0);
         if (chr == "D")
         {
            QTextStream s(&l_str, IO_ReadOnly);
            s >> f_tag;
            s >> f_dat;
            s >> f_odo;
            s >> len;
            s >> ws;
            while (len)
            {
               s >> chr;
               f_act.append(chr);
               len--;
            }

            // Dump all entries that are not the correct activity
            if ((f_dat == dat) && (f_odo == odo) && (f_act == act))
            {
               void;
            }
            else
               lst.append(l_str);
         }
         else
            lst.append(l_str);
      }
      o_f.close();
      o_f.remove();

      // Dump the entries to the original file
      if (o_f.open(IO_WriteOnly))
      {
         QTextStream tt(&o_f);
         for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it )
            tt << (*it) << endl;

         o_f.close();
      }
   }
}


// This function deletes an alert
void
mileagefile::deleteAlert(
   QString  odo,
   QString  rep,
   QString  act)
{
   QFile o_f(*_filestr);
   QStringList  lst;
   QString  f_tag, f_rep, f_odo;
   int      len;

   // Open file to read 
   if (o_f.open(IO_ReadOnly))
   {
      QTextStream t(&o_f);

      // Read all entries
      while (!t.atEnd())
      {
         QString  f_act;
         QString  l_str = t.readLine();
         QChar chr = l_str.at(0);
         if (chr == "C")
         {
            QTextStream s(&l_str, IO_ReadOnly);
            s >> f_tag;
            s >> f_odo;
            s >> f_rep;
            s >> len;
            s >> ws;
            while (len)
            {
               s >> chr;
               f_act.append(chr);
               len--;
            }

            // Dump all entries that are not the correct alert
            if ((f_odo == odo) && (f_rep == rep) && (f_act == act))
            {
               void;
            }
            else
               lst.append(l_str);
         }
         else
            lst.append(l_str);
      }
      o_f.close();
      o_f.remove();

      // Dump the entries to the original file
      if (o_f.open(IO_WriteOnly))
      {
         QTextStream tt(&o_f);
         for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it )
            tt << (*it) << endl;

         o_f.close();
      }
   }
}


// This function deletes a trip entry. It looks a match of the start 
// odometer.
void
mileagefile::deleteTrip(
   QString  s_odo)
{
   QFile o_f(*_filestr);
   QStringList  lst;
   int      len;

   // Open file to read
   if (o_f.open(IO_ReadOnly))
   {
      QTextStream t(&o_f);

      // Read all entries
      while (!t.atEnd())
      {
         QString  f_act;
         QString  l_str = t.readLine();
         QChar chr = l_str.at(0);
         if (chr == "F")
         {
            QString  f_tag, f_s_dat, f_s_odo;
            QTextStream s(&l_str, IO_ReadOnly);
            s >> f_tag;

            s >> len;
            s >> ws;
            while (len)
            {
               s >> chr;
               f_s_dat.append(chr);
               len--;
            }

            s >> len;
            s >> ws;
            while (len)
            {
               s >> chr;
               f_s_odo.append(chr);
               len--;
            }

            // Dump all entries that are not the correct trip
            if (f_s_odo == s_odo)
            {
               void;
            }
            else
               lst.append(l_str);
         }
         else
            lst.append(l_str);
      }
      o_f.close();
      o_f.remove();

      // Dump the entries to the original file
      if (o_f.open(IO_WriteOnly))
      {
         QTextStream tt(&o_f);
         for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it )
            tt << (*it) << endl;

         o_f.close();
      }
   }
}


// This searches to see if a car entry already exists
bool
mileagefile::searchForCar(const char *carname)
{
   QString dir_str;
   QDir    dir;
   bool return_val = FALSE;
   QString car_str;
   QFileInfo *fi;


   // Get directory
   if (getenv("HOME"))
   {
      dir_str = getenv("HOME");
      dir_str.append("/");
   }
   dir_str.append(CAR_DATA_DIR);
   dir.cd(dir_str);
   dir.setFilter(QDir::Files | QDir::NoSymLinks);
   const QFileInfoList *list = dir.entryInfoList();
   QFileInfoListIterator it( *list );
   car_str.append(carname);
   car_str.append(CAR_DATA_EXT);

   // Loop thru all files in directory
   while ( (fi = it.current()) != NULL )
   {
      if (fi->fileName() == car_str)
      {
         return_val = TRUE;
         break;
      }
      ++it;
   }
   return (return_val);
}


// This will create a new car file
bool
mileagefile::createNewCar(const char *carname)
{
   QString dir_str;
   bool return_val = FALSE;
   QString str;

   // Create directory and file
   if (getenv("HOME"))
   {
      dir_str = getenv("HOME");
      dir_str.append("/");
   }
   dir_str.append(CAR_DATA_DIR);
   QDir * dir = new QDir();
   dir->mkdir(dir_str, TRUE);
   delete dir;
   str.append(dir_str);
   str.append(carname);
   str.append(CAR_DATA_EXT);

   // Create file and write identity string
   QFile  filename(str);
   if (filename.open(IO_ReadWrite))
   {
      QTextStream t(&filename);
      t << FIRST_LINE_STRING << endl;
      t << "A " << carname << endl;

      filename.close();
      return_val = TRUE;
   }
   return (return_val);
}


// This function adds a new mileage entry
void
mileagefile::addMileEntry(
   QString str,
   bool    previous)
{
   QFile filename(*_filestr);

   // Open file to write at end
   if (filename.open(IO_ReadWrite | IO_Append))
   {
      // write line to file
      QTextStream t(&filename);
      if (previous)
         t << "P " << str << endl;
      else
         t << "M " << str << endl;

      filename.close();
   }
}

void
mileagefile::addAlertEntry(
   QString& odo_str,
   QString& repeat,
   QString& act_str,
   QString& comment_str)
{
   QFile filename(*_filestr);

   // Open file to write at end
   if (filename.open(IO_ReadWrite | IO_Append))
   {
      // write line to file
      QTextStream t(&filename);
      t << "C " << odo_str << " " << repeat << " ";
      t << act_str.length() << " " << act_str << " ";
      t << comment_str.length() << " " << comment_str << endl;

      filename.close();
   }
}


void
mileagefile::addActivityEntry(
   QString& date_str,
   QString& odo_str,
   QString& act_str,
   QString& comment_str)
{
   QFile filename(*_filestr);

   // Open file to write at end
   if (filename.open(IO_ReadWrite | IO_Append))
   {
      // write line to file
      QTextStream t(&filename);
      t << "D " << date_str << " " << odo_str << " ";
      t << act_str.length() << " " << act_str << " ";
      t << comment_str.length() << " " << comment_str << endl;

      filename.close();
   }
}


// This returns all mileage entries
void
mileagefile::returnMileEntries(QStringList & lst)
{
   QFile filename(*_filestr);
   QString  n_str;

   // Open the file for reading
   if (filename.open(IO_ReadOnly))
   {
      QTextStream t(&filename);

      while (!t.atEnd())
      {
         QString  str = t.readLine();
         QChar chr = str.at(0);
         if (chr == "M")
         {
            n_str = str.right(str.length()-2);
            n_str.append(" 0");
            lst << n_str;
         }
         else if (chr == "P")
         {
            n_str = str.right(str.length()-2);
            n_str.append(" 1");
            lst << n_str;
         }
      }
      filename.close();
   }
}


// This returns all activity entries
void
mileagefile::returnActivityEntries(QStringList & lst)
{
   returnEntries(lst, 'D');
}


// This returns all alert entries
void
mileagefile::returnAlertEntries(QStringList & lst)
{
   returnEntries(lst, 'C');
}


void
mileagefile::returnTripEntries(QStringList & lst)
{
   returnEntries(lst, 'F');
}


void
mileagefile::returnEntries(
   QStringList & lst,
   QChar  value)
{
   QFile filename(*_filestr);
   QString  n_str;

   // Open the file for reading
   if (filename.open(IO_ReadOnly))
   {
      QTextStream t(&filename);

      while (!t.atEnd())
      {
         QString  str = t.readLine();
         QChar chr = str.at(0);
         if (chr == value)
         {
            n_str = str.right(str.length()-2);
            lst << n_str;
         }
      }
      filename.close();
   }
}


void
mileagefile::getNextAlert(
   int           delta,
   QStringList & alert_lst)
{
   int  max_odo = -1;
   int  min_odo = 0;
   QFile filename(*_filestr);
   QString  n_str;


   // Open the file for reading
   if (filename.open(IO_ReadOnly))
   {
      QTextStream t(&filename);

      // Find min odometer value
      while (!t.atEnd())
      {
         QString  str = t.readLine();
         QChar chr = str.at(0);
         if ((chr == "M") || (chr == "P"))
         {
            QString  m_str, date_str;
            int      odo_int;
            QTextStream s(&str, IO_ReadOnly);

            s >> m_str >> date_str >> odo_int;
            if (odo_int > min_odo)
               min_odo = odo_int;
         }
      }

      // Set max range
      max_odo = min_odo + delta;
      filename.close();
   }

   // Open the file for reading
   if (filename.open(IO_ReadOnly))
   {
      // Find next alert to display
      QTextStream t1(&filename);
      while (!t1.atEnd())
      {
         QString  str = t1.readLine();
         QChar chr = str.at(0);
         if (chr == "C")
         {
            QString  m_str;
            int      odo_int, rep_int;
            QTextStream s(&str, IO_ReadOnly);

            s >> m_str >> odo_int >> rep_int;
            if (rep_int == 0)
            {
               // We are looking for all the odometer readings between
               // the last fillup and delta odometer readings
               if ((odo_int <= max_odo) && (odo_int >= min_odo))
               {
                  QString  rst;
                  rst.setNum(odo_int);

                  while (!s.atEnd())
                  {
                     QString s1;
                     s >> s1;
                     rst.append(" ");
                     rst.append(s1);
                  }
                  // Send odometer reading, activity, comment
                  alert_lst << rst;
               }
            }
            else
            {
               // First check to see if base odometer reading is below current
               if (odo_int <= max_odo)
               {
                  // Now fine the base or repeating value closest to current
                  int mult = 0;
                  int new_odo;
                  do
                  {
                     if ((mult * rep_int + odo_int) > min_odo)
                        break;
                     mult++;
                  }
                  while (TRUE);
                  new_odo = mult * rep_int + odo_int;

                  // IF this value between delta and max_odo
                  if (new_odo <= max_odo)
                  {
                     QString  rst;
                     rst.setNum(new_odo);

                     while (!s.atEnd())
                     {
                        QString s1;
                        s >> s1;
                        rst.append(" ");
                        rst.append(s1);
                     }
                     // Send odometer reading, activity, comment
                     alert_lst << rst;
                  }
               }
            }
         }
      }
      filename.close();
   }
}


void
mileagefile::addTripEntry(
   QString& start_date,
   QString& start_miles,
   QString& end_date,
   QString& end_miles,
   QString& person,
   QString& dest,
   QString& purpose)
{
   QFile filename(*_filestr);

   // Remove the old entry if it exists
   deleteTrip(start_miles);

   // Open file to write at end
   if (filename.open(IO_ReadWrite | IO_Append))
   {
      // write line to file
      QTextStream t(&filename);
      t << "F ";
      t << start_date.length() << " " << start_date << " ";
      t << start_miles.length() << " " << start_miles << " ";
      t << end_date.length() << " " << end_date << " ";
      t << end_miles.length() << " " << end_miles << " ";
      t << person.length() << " " << person << " ";
      t << dest.length() << " " << dest << " ";
      t << purpose.length() << " " << purpose << endl;

      filename.close();
   }
}


