/*
 * Copyright: (C) 2003 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@tns.net
 *
 */


#include "mileagegraph.h"
#include "mileagegraphdata.h"

#include <qwidget.h>
#include <qpainter.h>
#include <qcombobox.h>
#include <qdatetime.h>
#include <qpen.h>

#include <math.h>

#include <cstdio>
#include <cstdlib>
#include <iostream>

#define X_OFFSET 35
#define X_DELTA  16
#define X_NUM    13
#define X_MAX_NUM X_NUM * X_DELTA + X_OFFSET

#define Y_OFFSET 20
#define Y_DELTA  16
#define Y_NUM    11
#define Y_MAX_NUM Y_NUM * Y_DELTA + Y_OFFSET

#define MILEAGE_STR   "Mileage"
#define DISTANCE_STR  "Distance"
#define COST_STR      "Cost"


// Constructor
mileagegraph::mileagegraph(QSortedList<mileagegraphdata>& lst, QWidget* parent, const char* name) : QWidget(parent,name), _list(lst)
{
   _units = 0;
   _first_year = 0;
   _last_year = 0;
   _max_display = 100;
   _max_cost = 4;

   createPens();
}

// Destructor
mileagegraph::~mileagegraph()
{
}


void
mileagegraph::setUnits(
   int units,
   int cost)
{
   _units = units;
   _cost = cost;

   _cost_str = "Cost/Unit";

   switch (units)
   {
      // MPG
      case 0:
         _dist = "M/Day";
         _mile = "MPG";
      break;

      // L/100km
      case 1:
         _dist = "Km/Day";
         _mile = "L/100km";
      break;

      // L/10km
      case 2:
         _dist = "Km/Day";
         _mile = "L/10km";
      break;

      // Miles / L
      case 3:
         _dist = "M/Day";
         _mile = "M/L";
      break;

      // km / L
      case 4:
         _dist = "Km/Day";
         _mile = "Km/L";
      break;
   }
}


void
mileagegraph::createPens()
{
   _pen_graph.setStyle(Qt::SolidLine);
   _pen_graph.setWidth(1);
   _pen_graph.setColor(Qt::black);

   for (int i = 0; i < 8; i++)
   {
      _pen[i].setStyle(Qt::SolidLine);
      _pen[i].setWidth(2);
      _pen[i].setColor(Qt::black);
   }
   _pen[0].setColor(Qt::red);
   _pen[1].setColor(Qt::blue);
   _pen[2].setColor(Qt::green);
   _pen[3].setColor(Qt::cyan);
   _pen[4].setColor(Qt::magenta);
   _pen[5].setColor(Qt::yellow);
   _pen[6].setColor(Qt::darkGray);
   _pen[7].setColor(Qt::darkBlue);
}


void
mileagegraph::paintEvent(QPaintEvent *e)
{
   QPainter  *p;

   p = new QPainter(this);

   // Erase entire display
   this->erase();
   drawLegend(-1, p);

   // Set pen for graph grid and draw grid
   p->setPen(_pen_graph);
   drawGrid(p);

   // Draw Text
   drawText(p);

   // Draw data onto graph
   drawData(p);

   delete p;
}


void
mileagegraph::drawGrid(QPainter *p)
{
   // Draw horizontal grid
   for (int y = 0; y < Y_NUM; y++)
      p->drawLine(X_OFFSET, Y_OFFSET + Y_DELTA * y,
                  X_OFFSET + (X_NUM - 1) * X_DELTA, Y_OFFSET + Y_DELTA * y);

   // Draw vertical grid
   for (int x = 0; x < X_NUM; x++)
      p->drawLine(X_OFFSET + X_DELTA * x, Y_OFFSET,
                  X_OFFSET + X_DELTA * x, Y_OFFSET + (Y_NUM - 1) * Y_DELTA);

   int label1 = (Y_NUM * Y_DELTA) / 2 + Y_OFFSET;
   if (_graphtype2 == MILEAGE_STR)
      p->drawText(0, label1, _mile);
   else if (_graphtype2 == DISTANCE_STR)
      p->drawText(0, label1, _dist);
   else if (_graphtype2 == COST_STR)
      p->drawText(0, label1, _cost_str);
}


void
mileagegraph::drawText(QPainter *p)
{
   QString  jan = "Jan";
   QString  apr = "Apr";
   QString  jul = "Jul";
   QString  oct = "Oct";
   QString  zero= "0";
   QString  one;
   QString  two;
   QString  three;
   QString  four;
   QString  five;


   if (_graphtype2 == MILEAGE_STR)
      _max_display = _max_mileage / 50 * 50 + 50;
   else if (_graphtype2 == DISTANCE_STR)
      _max_display = _max_dist / 50 * 50 + 50;
   else if (_graphtype2 == COST_STR)
      _max_display = _max_cost;
   else
      _max_display = 100;

   one.setNum(_max_display / 10.0 * 2.0, 'g', 4);
   two.setNum(_max_display / 10.0 * 4.0, 'g', 4);
   three.setNum(_max_display / 10.0 * 6.0, 'g', 4);
   four.setNum(_max_display / 10.0 * 8.0, 'g', 4);
   five.setNum(_max_display);

   p->drawText(X_OFFSET + 0 * X_DELTA - (X_DELTA / 2),
               Y_OFFSET + (Y_NUM * Y_DELTA), jan);
   p->drawText(X_OFFSET + 3 * X_DELTA - (X_DELTA / 2),
               Y_OFFSET + (Y_NUM * Y_DELTA), apr);
   p->drawText(X_OFFSET + 6 * X_DELTA - (X_DELTA / 2),
               Y_OFFSET + (Y_NUM * Y_DELTA), jul);
   p->drawText(X_OFFSET + 9 * X_DELTA - (X_DELTA / 2),
               Y_OFFSET + (Y_NUM * Y_DELTA), oct);
   p->drawText(X_OFFSET + 12 * X_DELTA - (X_DELTA / 2),
               Y_OFFSET + (Y_NUM * Y_DELTA), jan);

   p->drawText(1, Y_OFFSET + ((Y_NUM-1) * Y_DELTA) + Y_DELTA/2, zero);
   p->drawText(1, Y_OFFSET + ((Y_NUM-3) * Y_DELTA) + Y_DELTA/2, one);
   p->drawText(1, Y_OFFSET + ((Y_NUM-5) * Y_DELTA) + Y_DELTA/2, two);
   p->drawText(1, Y_OFFSET + ((Y_NUM-7) * Y_DELTA) + Y_DELTA/2, three);
   p->drawText(1, Y_OFFSET + ((Y_NUM-9) * Y_DELTA) + Y_DELTA/2, four);
   p->drawText(1, Y_OFFSET + ((Y_NUM-11) * Y_DELTA) + Y_DELTA/2, five);
}


void
mileagegraph::drawData(QPainter *p)
{
   int  i, x;


   if (_graphtype == "All Years")
   {
      for (i = _first_year, x = 0; i <= _last_year; i++, x++)
      {
         p->setPen(_pen[x%8]);
         drawLegend(i, p);
         graphIt(i, p);
      }
   }
   else if (_graphtype.right(4) == "Year")
   {
      int num;

      num = _graphtype.left(4).toInt();
      p->setPen(_pen[0]);
      drawLegend(num, p);
      graphIt(num, p);
   }
}


void
mileagegraph::drawLegend(
   int year,
   QPainter  *p)
{
   QString  str;
   static int      offset = 0;

   if (year == -1)
      offset = 0;
   else
   {
      str.setNum(year);
      p->drawText(offset, Y_OFFSET / 2, str);
      offset += 30;
   }
}


void
mileagegraph::graphMileageYear(
   int year,
   QPainter  *p)
{
   mileagegraphdata  *data;
   int  prev_odo = -1;
   double prev_fuel = 0.0;
   double  mpg;
   int     x_val, y_val;
   int    p_x = -1;
   int    p_y = -1;
   int    y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;


   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      if (data->getDate().year() == year)
      {
         if (data->getPrevValue() == FALSE)
         {
         }
         else if (prev_odo == -1)
         {
         }
         else
         {
            if (_units == 0)
               mpg = (data->getOdometer() - prev_odo) / data->getFuel();
            else if (_units == 1)
               mpg = 100.0 * data->getFuel() / (data->getOdometer() - prev_odo);
            else if (_units == 2)
               mpg = 10.0 * data->getFuel() / (data->getOdometer() - prev_odo);
            else if (_units == 3)
               mpg = (data->getOdometer() - prev_odo) / data->getFuel();
            else if (_units == 4)
               mpg = (data->getOdometer() - prev_odo) / data->getFuel();
            else
               mpg = (data->getOdometer() - prev_odo) / data->getFuel();

            y_val = y_max - (int)(mpg * ((y_max - Y_OFFSET) / (double)_max_display));
            x_val = (data->getDate().month() - 1) * X_DELTA + X_OFFSET;
            x_val = x_val + (X_DELTA * data->getDate().day() / data->getDate().daysInMonth());

            if (p_x != -1)
            {
               if (x_val > X_MAX_NUM)
                  x_val = X_MAX_NUM;
               if (x_val < 0)
                  x_val = 0;
               if (y_val > Y_MAX_NUM)
                  y_val = Y_MAX_NUM;
               if (y_val < 0)
                  y_val = 0;
               p->drawLine(p_x, p_y, x_val, y_val);
            }
            p_x = x_val;
            p_y = y_val;
         }
      }
      prev_odo = data->getOdometer();
      prev_fuel= data->getFuel();
   }
}


void
mileagegraph::graphDistanceYear(
   int year,
   QPainter  *p)
{
   mileagegraphdata  *data;
   mileagegraphdata  *prev_data = NULL;
   int                miles;
   int                prev_odo = -1;
   int                days;
   int     x_val, y_val;
   int    p_x = -1;
   int    p_y = -1;
   int    y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;


   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      if (data->getDate().year() == year)
      {
         if (data->getPrevValue() == FALSE)
         {
         }
         else if (prev_odo == -1)
         {
         }
         else
         {
            miles = data->getOdometer() - prev_odo;
            days = prev_data->getDate().daysTo(data->getDate());
            // IF fillup on same day then days to 1
            if (days == 0)
               days = 1;
            y_val = y_max - (int)((double)miles / (double)days * (y_max - Y_OFFSET) / (double)_max_display);
            x_val = (data->getDate().month() - 1) * X_DELTA + X_OFFSET;
            x_val = x_val + (X_DELTA * data->getDate().day() / data->getDate().daysInMonth());

            if (p_x != -1)
            {
               if (x_val > X_MAX_NUM)
                  x_val = X_MAX_NUM;
               if (x_val < 0)
                  x_val = 0;
               if (y_val > Y_MAX_NUM)
                  y_val = Y_MAX_NUM;
               if (y_val < 0)
                  y_val = 0;
               p->drawLine(p_x, p_y, x_val, y_val);
            }
            p_x = x_val;
            p_y = y_val;
         }
      }
      prev_odo = data->getOdometer();
      prev_data = data;
   }
}


void
mileagegraph::graphCostYear(
   int year,
   QPainter  *p)
{
   mileagegraphdata  *data;
   mileagegraphdata  *prev_data = NULL;
   double             cost;
   int     x_val, y_val;
   int    p_x = -1;
   int    p_y = -1;
   int    y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;


   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      if (data->getDate().year() == year)
      {
         if (data->getPrevValue() == FALSE)
         {
         }
         else if (prev_data == NULL)
         {
         }
         else
         {
            if (_cost == 0)
            {
               if (data->getFuel() == 0)
                  cost = 0;
               else
                  cost = data->getCost() / data->getFuel();
            }
            else
               cost = data->getCost();

            y_val = y_max - (int)(cost * (y_max - Y_OFFSET) / (double)_max_display);
            x_val = (data->getDate().month() - 1) * X_DELTA + X_OFFSET;
            x_val = x_val + (X_DELTA * data->getDate().day() / data->getDate().daysInMonth());

            if (p_x != -1)
            {
               if (x_val > X_MAX_NUM)
                  x_val = X_MAX_NUM;
               if (x_val < 0)
                  x_val = 0;
               if (y_val > Y_MAX_NUM)
                  y_val = Y_MAX_NUM;
               if (y_val < 0)
                  y_val = 0;
               p->drawLine(p_x, p_y, x_val, y_val);
            }
            p_x = x_val;
            p_y = y_val;
         }
      }
      prev_data = data;
   }
}


int
mileagegraph::getMaxDistance()
{
   mileagegraphdata  *data;
   mileagegraphdata  *prev_data = NULL;
   int                miles;
   int                prev_odo = -1;
   int                days;
   int                dist_per_day = 0;
   int                temp;


   // Loop thru all distance data and find greatest to set scale
   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      if (data->getPrevValue() == FALSE)
      {
      }
      else if (prev_odo == -1)
      {
      }
      else
      {
         miles = data->getOdometer() - prev_odo;
         days = prev_data->getDate().daysTo(data->getDate());
         // IF fillup on same day then set days to 1
         if (days == 0)
            days = 1;
         temp = (int)((double)miles / (double)days);
         if (temp > dist_per_day)
            dist_per_day = temp;
      }
      prev_odo = data->getOdometer();
      prev_data = data;
   }
   return (dist_per_day);
}


int
mileagegraph::getMaxCost()
{
   mileagegraphdata  *data;
   int                max_cost = -1;
   int                cost;


   // Loop thru all cost data and find greatest to set scale
   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      // IF Fillup
      if (_cost == 0)
      {
         if (data->getFuel() == 0)
            cost = 0;
         else
            cost = (int)ceil(data->getCost() / data->getFuel());
         if (cost > max_cost)
            max_cost = cost;
      }
      // ELSE per unit
      else
      {
         if (data->getCost() > max_cost)
            max_cost = (int)ceil(data->getCost());
      }
   }
   return (max_cost);
}


int
mileagegraph::getMaxMileage()
{
   mileagegraphdata  *data;
   int  prev_odo = -1;
   double prev_fuel = 0.0;
   double mpg = 0.0;
   double temp;


   // Loop thru all MPG data and find greatest to set scale
   for ( data = _list.first(); data != 0; data = _list.next() )
   {
      if (data->getPrevValue() == FALSE)
      {
      }
      else if (prev_odo == -1)
      {
      }
      else
      {
         if (_units == 0)
            temp = (data->getOdometer() - prev_odo) / data->getFuel();
         else if (_units == 1)
            temp = 100.0 * data->getFuel() / (data->getOdometer() - prev_odo);
         else if (_units == 2)
            temp = 10.0 * data->getFuel() / (data->getOdometer() - prev_odo);
         else if (_units == 3)
            temp = (data->getOdometer() - prev_odo) / data->getFuel();
         else if (_units == 4)
            temp = (data->getOdometer() - prev_odo) / data->getFuel();
         else
            temp = (data->getOdometer() - prev_odo) / data->getFuel();

         if (temp > mpg)
            mpg = temp;
      }
      prev_odo = data->getOdometer();
      prev_fuel= data->getFuel();
   }
   return((int)mpg);
}


void
mileagegraph::graphIt(
   int year,
   QPainter  *p)
{
   if (_graphtype2 == MILEAGE_STR)
      graphMileageYear(year, p);
   else if (_graphtype2 == DISTANCE_STR)
      graphDistanceYear(year, p);
   else if (_graphtype2 == COST_STR)
      graphCostYear(year, p);
}


void
mileagegraph::currentGraphChanged(const QString& str)
{
   _graphtype = str;
}


void
mileagegraph::currentGraph2Changed(const QString& str)
{
   _graphtype2 = str;
}


void
mileagegraph::currentCarChanged(QComboBox *year)
{
   _list.sort();
   year->clear();

   _graphtype = "No Graph";
   year->insertItem("No Graph");
   year->insertItem("All Years");

   _first_year = 0;
   _last_year = 0;
   if (_list.first())
      _first_year = _list.first()->getDate().year();
   if (_list.last())
      _last_year = _list.last()->getDate().year();

   // Output a menu item for each years data
   if (_first_year != 0)
   {
      for (int i = _first_year; i <= _last_year; i++)
      {
         QString  str;

         str.setNum(i);
         str.append(" Year");
         year->insertItem(str);
      }
   }

   _max_dist = getMaxDistance();
   _max_mileage = getMaxMileage();
   _max_cost = getMaxCost();

   this->repaint();
}



